from components.Timezone import datetime_by_tzone
from enums.ProcessStatusEnum import ProcessStatusEnum
import logging
import json
from io import StringIO
from typing import Dict, Any, List
from components.S3Route import get_file_from_prefix

logger = logging.getLogger()


def get_tasks_from_control(conf: List[Dict[str, Any]], type_task: str) -> Dict[str, Any]:
    response = {'status': ProcessStatusEnum.SUCCESS.value, 'tasks': [], 'reset': False}
    try:
        conf = conf[-1]
        status = conf["status"]
        if "reset_by_user" in conf.keys():
            response["reset"] = True
        if status == ProcessStatusEnum.FAIL.value:
            response["status"] = ProcessStatusEnum.FAIL.value
            success_tasks = conf["tasks"]
            final_tasks = []
            for key in success_tasks.keys():
                if type_task.lower() not in key.lower():
                    continue
                task = success_tasks[key]
                task_status = task["status"]
                if task_status == ProcessStatusEnum.SUCCESS.value:
                    command = task["description"]
                    result = (key, command)
                    final_tasks.append(result)
            response["tasks"] = final_tasks
    except Exception as e:
        logger.error(f"Error obteniendo task fallidos desde control. {e}")
    finally:
        return response


def update_new_process(conf: List[Dict[str, Any]], status: str, tasks: Dict[str, Any],
                       timezone: str, task,  created_tables: List, delete_last: bool = False, frequency: str = "montly") -> List[Dict[str, Any]]:
    try:
        format_date = "%Y-%m" if frequency == "montly" else "%Y-%W"
        current_period = str(datetime_by_tzone(timezone, format_date))[:7]
        processed_period = task.xcom_pull(task_ids="SCRIPTS-EXTRACTOR", key="PROCESSED_PERIOD")
        if delete_last:
            last_tasks = conf[-1]["tasks"]
            tasks.update(last_tasks)
            conf.pop(-1)
        current_datetime = str(datetime_by_tzone(timezone, '%Y-%m-%d %H:%M:%S'))
        new_process = {"date": current_datetime, "status": status, "tasks": tasks, "objects_created": created_tables}
        if current_period == processed_period and isinstance(conf, List):
            conf.append(new_process)
        else:
            conf = [new_process]
    except Exception as e:
        logger.error(f"Error actualizando archivo de control. {e}")
    finally:
        return conf


def extract_last_control(conn_id: str, bucket: str, prefix: str, provider: str, timezone: str, **kwargs):
    try:
        task = kwargs['ti']
        if not prefix.endswith("/"):
            prefix += "/"
        logger.info(f"BUSCANDO Y EXTRAYENDO ARCHIVO DE CONTROL DESDE {prefix}")
        control, control_key = get_file_from_prefix(conn_id, bucket, prefix, provider, timezone, task)
        if control:
            str_data = str(control.getvalue(), encoding='UTF-8', errors='ignore')
            data = StringIO(str_data)
            if not data:
                raise AssertionError("Archivo de control no tiene un formato correcto")
            else:
                data = json.load(data)
                task.xcom_push(key="CONTROL-CONFIG", value=[control_key, data])
        else:
            raise AssertionError("Archivo de control no encontrado")
    except Exception as e:
        logger.error(f"Error general de descarga de archivo de control. {e}")