from typing import Any, Dict

import pandas as pd
import logging

logger = logging.getLogger()


def get_steps(sql_command: str, chunksize: int, connection, is_tablename: bool = False) -> Dict[str, Any]:
    response = {'steps': 0, 'error': ''}
    try:
        if is_tablename:
            count_command = f'SELECT COUNT(*) FROM {sql_command}'
        else:
            count_command = f"SELECT COUNT(*) FROM ({sql_command}) BCOM"
        with connection.connect() as conn:
            result = conn.execute(count_command).first()
            if result:
                total_rows = int(result[0])
        if total_rows == chunksize:
            response["steps"] = 1
        else:
            response["steps"] = int(total_rows/chunksize) + 1
    except Exception as e:
        message = f"Error calculando el total de N° de filas desde el comando: {sql_command}. {e}"
        logger.error(message)
        response["error"] = message
    finally:
        return response


def get_iterator(command: str, chunksize: int, connection) -> Dict[str, Any]:
    response = {'iterator': None, 'error': ''}
    try:
        connection = connection.execution_options(stream_results=True)
        iterator = pd.read_sql(command, connection, index_col=None, chunksize=chunksize)
        response["iterator"] = iter(iterator)
    except Exception as e:
        message = f"Error trayendo iterator. {e}"
        logger.error(message)
        response["error"] = message
    finally:
        return response

